'use strict';

var through = require('through2');
var path = require('path');
var fs = require('fs');
var uglifyjs = require('uglify-js');
var replacepath = require('replacepath');
var rev = require('./rev');
var mod = require('./module');

var unique = function(arr){
    var newarr = [];
    var temp = {};
    arr.forEach(function(val){
        if(!temp[val]){
            temp[val] = true
            newarr.push(val)
        }
    })
    return newarr
}

module.exports = function(opts){
    var options = {
        //路径别名
        paths:{},
        //模块别名
        alias:{},
        //忽略的模块名，模块不会追加到合并的文件中
        ignore:['util', 'template', 'component'],
        //版本号长度
        verlen:7,
        //生成合并css文件
        cssdebug:false,
        //生成合并css压缩文件
        cssmin:true,
        //生成合并js文件
        jsdebug:false,
        //生成合并js压缩文件
        jsmin:true,
        //过滤资源文件路径，去除或替换一些特殊符号，返回绝对路径
        filterPath:null,
        //gulp-watch返回的对象
        watcher:null,
        //只输出改变过的文件
        changed:true
    }

    if(opts){
        for(var i in opts){
            options[i] = opts[i]
        }
    }

    if(options.jsmin === true){
        options.jsmin = {};
    }

    if(options.jsmin.fromString !== true){
        options.jsmin.fromString = true
    }

    if(options.paths.base){
        for(var i in options.paths){
            var _path = options.paths[i];
            if(i !== 'base' && !/^[A-Z]:/.test(_path)){
                options.paths[i] = options.paths.base + _path;
            }
        }
    }

    return through.obj(function(file, enc, cb){

        if(file.isNull()){
	        return cb();
        }

        var dirname = path.dirname(file.path)+'/';
        var content = file.contents.toString().replace(/^\s+|\s+$/g, '');

        if(!content){
            return cb();
        }

        var matchs = content.match(/Nui.load\(['"][^'"]+['"]/g);

        if(matchs){
            var Module = mod(dirname, options);
            matchs.forEach(function(val, i){
                var id = matchs[i].replace(/Nui.load\(/, '').replace(/['"]/g, '').replace(/-debug|-min/g, '');
                Module.load(id, function(mod){
                    var contents = unique(mod.getContents()).join('\n');
                    var styles = Module.getStyles(unique(mod.loadstyles())).join('\n');
                    var url;
                    var filename;
                    for(var i in mod.depmodules){
                        url = mod.depmodules[i].url;
                        filename = mod.depmodules[i].filename
                        break;
                    }
                    if(fs.existsSync(url)){
                        //将重复模块但是模块名不同替换为已存在的模块
                        for(var i in Module.repeatMap){
                            if(i){
                                Module.repeatMap[i].forEach(function(v){
                                    v = v.replace(/[\W]/g, function(str){
                                        return '\\'+str
                                    });
                                    contents = contents.replace(new RegExp('([\'\"])'+ v +'([\'\"])', 'g'), '$1'+i+'$2');
                                })
                            }
                        }
                        
                        if(styles){
                            var _path = path.join(path.dirname(url), '../style/')
                            if(!fs.existsSync(_path)){
                                fs.mkdirSync(_path)
                            }
                            var cssmin = _path + filename + '-min.css';
                            var cssdebug = _path + filename + '-debug.css';
                            if(options.cssdebug === true){
                                fs.writeFileSync(cssdebug, styles);
                            }
                            else if(fs.existsSync(cssdebug)){
                                fs.unlinkSync(cssdebug)
                            }
                            if(options.cssmin === true){
                                styles = styles.replace(/\s*([\}\{\:\;\(\)\=\~\>\+\[\]\,])\s*/g, '$1')
                                               .replace(/\;\}/g, '}')
                                               .replace(/\/\*[^\/]*\*\//g, '')
                                               .replace(/\s{2,}/, ' ');
                                fs.writeFileSync(cssmin, styles);
                            }
                            else if(fs.existsSync(cssmin)){
                                fs.unlinkSync(cssmin)
                            }
                        }

                        var jsmin = url.replace(/(\.js)$/, '-min$1');
                        var jsdebug = url.replace(/(\.js)$/, '-debug$1');
                        if(options.jsdebug === true){
                            fs.writeFileSync(jsdebug, contents);
                        }
                        else if(fs.existsSync(jsdebug)){
                            fs.unlinkSync(jsdebug)
                        }
                        if(typeof options.jsmin === 'object'){
                            contents = uglifyjs.minify(contents, options.jsmin).code;
                            fs.writeFileSync(jsmin, contents);
                        }
                        else if(fs.existsSync(jsmin)){
                            fs.unlinkSync(jsmin)
                        }
                    }
                })
            })
        }

        if(options.verlen > 0){
            var newcon = rev.call(Module, content, dirname, options);
            if(options.changed === true && newcon === content){
                return cb()
            }
            file.contents = new Buffer(newcon);
        }

        this.push(file);
        cb();
    })
}
